@@ -47,22 +47,23 @@ const _PDF_LOADER_TIMEOUT = 400;
 
 const SpinnerBox = new Lang.Class({
     Name: 'SpinnerBox',
+    Extends: Gtk.Grid,
 
     _init: function() {
-        this.widget = new Gtk.Grid({ orientation: Gtk.Orientation.VERTICAL,
-                                     row_spacing: 24,
-                                     hexpand: true,
-                                     vexpand: true,
-                                     halign: Gtk.Align.CENTER,
-                                     valign: Gtk.Align.CENTER });
+        this.parent({ orientation: Gtk.Orientation.VERTICAL,
+                      row_spacing: 24,
+                      hexpand: true,
+                      vexpand: true,
+                      halign: Gtk.Align.CENTER,
+                      valign: Gtk.Align.CENTER });
 
         this._spinner = new Gtk.Spinner({ width_request: _ICON_SIZE,
                                           height_request: _ICON_SIZE,
                                           halign: Gtk.Align.CENTER,
                                           valign: Gtk.Align.CENTER });
-        this.widget.add(this._spinner);
+        this.add(this._spinner);
 
-        this.widget.show_all();
+        this.show_all();
     },
 
     start: function() {
@@ -76,14 +77,14 @@ const SpinnerBox = new Lang.Class({
 
 const Embed = new Lang.Class({
     Name: 'Embed',
+    Extends: Gtk.Box,
 
     _init: function() {
-        this._noResultsChangeId = 0;
         this._loadShowId = 0;
         this._searchState = null;
 
-        this.widget = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL,
-                                    visible: true });
+        this.parent({ orientation: Gtk.Orientation.VERTICAL,
+                      visible: true });
 
         let toplevel = Application.application.get_windows()[0];
         this._titlebar = new Gtk.Grid({ visible: true });
@@ -91,10 +92,10 @@ const Embed = new Lang.Class({
 
         // create the toolbar for selected items, it's hidden by default
         this._selectionToolbar = new Selections.SelectionToolbar();
-        this.widget.pack_end(this._selectionToolbar.widget, false, false, 0);
+        this.pack_end(this._selectionToolbar, false, false, 0);
 
         this._stackOverlay = new Gtk.Overlay({ visible: true });
-        this.widget.pack_end(this._stackOverlay, true, true, 0);
+        this.pack_end(this._stackOverlay, true, true, 0);
 
         this._stack = new Gtk.Stack({ visible: true,
                                       homogeneous: true,
@@ -106,22 +107,22 @@ const Embed = new Lang.Class({
 
         // now create the actual content widgets
         this._documents = new View.ViewContainer(WindowMode.WindowMode.DOCUMENTS);
-        this._stack.add_titled(this._documents.widget, 'documents', _("Recent"));
+        this._stack.add_titled(this._documents, 'documents', _("Recent"));
 
         this._collections = new View.ViewContainer(WindowMode.WindowMode.COLLECTIONS);
-        this._stack.add_titled(this._collections.widget, 'collections', _("Collections"));
+        this._stack.add_titled(this._collections, 'collections', _("Collections"));
 
         this._search = new View.ViewContainer(WindowMode.WindowMode.SEARCH);
-        this._stack.add_named(this._search.widget, 'search');
+        this._stack.add_named(this._search, 'search');
 
         this._preview = new Preview.PreviewView(this._stackOverlay);
-        this._stack.add_named(this._preview.widget, 'preview');
+        this._stack.add_named(this._preview, 'preview');
 
-        this._edit = new Edit.EditView(this._stackOverlay);
-        this._stack.add_named(this._edit.widget, 'edit');
+        this._edit = new Edit.EditView();
+        this._stack.add_named(this._edit, 'edit');
 
         this._spinnerBox = new SpinnerBox();
-        this._stack.add_named(this._spinnerBox.widget, 'spinner');
+        this._stack.add_named(this._spinnerBox, 'spinner');
 
         this._stack.connect('notify::visible-child',
                             Lang.bind(this, this._onVisibleChildChanged));
@@ -231,8 +232,8 @@ const Embed = new Lang.Class({
     },
 
     _onFullscreenChanged: function(controller, fullscreen) {
-        this._toolbar.widget.visible = !fullscreen;
-        this._toolbar.widget.sensitive = !fullscreen;
+        this._toolbar.visible = !fullscreen;
+        this._toolbar.sensitive = !fullscreen;
     },
 
     _onSearchChanged: function() {
@@ -270,9 +271,9 @@ const Embed = new Lang.Class({
         let visibleChild = this._stack.visible_child;
         let windowMode = WindowMode.WindowMode.NONE;
 
-        if (visibleChild == this._collections.widget)
+        if (visibleChild == this._collections)
             windowMode = WindowMode.WindowMode.COLLECTIONS;
-        else if (visibleChild == this._documents.widget)
+        else if (visibleChild == this._documents)
             windowMode = WindowMode.WindowMode.DOCUMENTS;
 
         if (windowMode == WindowMode.WindowMode.NONE)
@@ -304,8 +305,8 @@ const Embed = new Lang.Class({
         }
 
         if (this._toolbar.searchbar) {
-            this._toolbar.searchbar.connect('activate-result',
-                                            Lang.bind(this, this._onActivateResult));
+            this._toolbar.searchbar.connectJS('activate-result',
+                                               Lang.bind(this, this._onActivateResult));
         }
     },
 
@@ -371,7 +372,7 @@ const Embed = new Lang.Class({
         docModel.set_page_layout(EvView.PageLayout.AUTOMATIC);
         this._toolbar.setModel(docModel);
         this._preview.setModel(docModel);
-        this._preview.widget.grab_focus();
+        this._preview.grab_focus();
 
         this._clearLoadTimer();
         this._spinnerBox.stop();
@@ -388,9 +389,9 @@ const Embed = new Lang.Class({
         this._spinnerBox.stop();
 
         let dialog = new Password.PasswordDialog(doc);
-        dialog.widget.connect('response', Lang.bind(this,
+        dialog.connect('response', Lang.bind(this,
             function(widget, response) {
-                dialog.widget.destroy();
+                dialog.destroy();
                 if (response == Gtk.ResponseType.CANCEL || response == Gtk.ResponseType.DELETE_EVENT)
                     Application.documentManager.setActiveItem(null);
             }));
@@ -425,11 +426,11 @@ const Embed = new Lang.Class({
 
         if (createToolbar) {
             if (this._toolbar)
-                this._toolbar.widget.destroy();
+                this._toolbar.destroy();
 
             // pack the toolbar
             this._toolbar = new MainToolbar.OverviewToolbar(this._stackOverlay, this._stack);
-            this._titlebar.add(this._toolbar.widget);
+            this._titlebar.add(this._toolbar);
         }
 
         this._spinnerBox.stop();
@@ -440,11 +441,11 @@ const Embed = new Lang.Class({
         if (this._edit)
             this._edit.setUri(null);
         if (this._toolbar)
-            this._toolbar.widget.destroy();
+            this._toolbar.destroy();
 
         // pack the toolbar
         this._toolbar = new Preview.PreviewToolbar(this._preview);
-        this._titlebar.add(this._toolbar.widget);
+        this._titlebar.add(this._toolbar);
 
         this._stack.set_visible_child_name('preview');
     },
@@ -453,11 +454,11 @@ const Embed = new Lang.Class({
         if (this._preview)
             this._preview.setModel(null);
         if (this._toolbar)
-            this._toolbar.widget.destroy();
+            this._toolbar.destroy();
 
         // pack the toolbar
         this._toolbar = new Edit.EditToolbar(this._preview);
-        this._titlebar.add(this._toolbar.widget);
+        this._titlebar.add(this._toolbar);
 
         this._stack.set_visible_child_name('edit');
     },
