@@ -37,18 +37,19 @@ const Utils = imports.utils;
 
 const Searchbar = new Lang.Class({
     Name: 'Searchbar',
+    Extends: Gtk.SearchBar,
 
     _init: function() {
         this.searchChangeBlocked = false;
 
-        this.widget = new Gtk.SearchBar();
+        this.parent();
 
         // subclasses will create this._searchEntry and this._searchContainer
         // GtkWidgets
         this.createSearchWidgets();
 
-        this.widget.add(this._searchContainer);
-        this.widget.connect_entry(this._searchEntry);
+        this.add(this._searchContainer);
+        this.connect_entry(this._searchEntry);
 
         this._searchEntry.connect('search-changed', Lang.bind(this,
             function() {
@@ -57,9 +58,9 @@ const Searchbar = new Lang.Class({
 
                 this.entryChanged();
             }));
-        this.widget.connect('notify::search-mode-enabled', Lang.bind(this,
+        this.connect('notify::search-mode-enabled', Lang.bind(this,
             function() {
-                let searchEnabled = this.widget.search_mode_enabled;
+                let searchEnabled = this.search_mode_enabled;
                 Application.application.change_action_state('search', GLib.Variant.new('b', searchEnabled));
             }));
 
@@ -68,20 +69,20 @@ const Searchbar = new Lang.Class({
             Lang.bind(this, this._onActionStateChanged));
         this._onActionStateChanged(Application.application, 'search', Application.application.get_action_state('search'));
 
-        this.widget.connect('destroy', Lang.bind(this,
+        this.connect('destroy', Lang.bind(this,
             function() {
                 Application.application.disconnect(searchStateId);
                 Application.application.change_action_state('search', GLib.Variant.new('b', false));
             }));
 
-        this.widget.show_all();
+        this.show_all();
     },
 
     _onActionStateChanged: function(source, actionName, state) {
         if (state.get_boolean())
-            this.show();
+            this.reveal();
         else
-            this.hide();
+            this.conceal();
     },
 
     createSearchWidgets: function() {
@@ -92,36 +93,32 @@ const Searchbar = new Lang.Class({
         log('Error: Searchbar implementations must override entryChanged');
     },
 
-    destroy: function() {
-        this.widget.destroy();
-    },
-
     handleEvent: function(event) {
         // Skip if the search bar is shown and the focus is elsewhere
-        if (this.widget.search_mode_enabled && !this._searchEntry.is_focus)
+        if (this.search_mode_enabled && !this._searchEntry.is_focus)
             return false;
 
         let keyval = event.get_keyval()[1];
-        if (this.widget.search_mode_enabled && keyval == Gdk.KEY_Return) {
-            this.emit('activate-result');
+        if (this.search_mode_enabled && keyval == Gdk.KEY_Return) {
+            this.emitJS('activate-result');
             return true;
         }
 
-        return this.widget.handle_event(event);
+        return this.handle_event(event);
     },
 
-    show: function() {
-        this.widget.search_mode_enabled = true;
+    reveal: function() {
+        this.search_mode_enabled = true;
     },
 
-    hide: function() {
-        this.widget.search_mode_enabled = false;
+    conceal: function() {
+        this.search_mode_enabled = false;
 
         // clear all the search properties when hiding the entry
         this._searchEntry.set_text('');
     }
 });
-Signals.addSignalMethods(Searchbar.prototype);
+Utils.addJSSignalMethods(Searchbar.prototype);
 
 const Dropdown = new Lang.Class({
     Name: 'Dropdown',
@@ -160,13 +157,13 @@ const OverviewSearchbar = new Lang.Class({
 
         this.parent();
 
-        this._sourcesId = Application.sourceManager.connect('active-changed',
+        let sourcesId = Application.sourceManager.connect('active-changed',
             Lang.bind(this, this._onActiveSourceChanged));
-        this._searchTypeId = Application.searchTypeManager.connect('active-changed',
+        let searchTypeId = Application.searchTypeManager.connect('active-changed',
             Lang.bind(this, this._onActiveTypeChanged));
-        this._searchMatchId = Application.searchMatchManager.connect('active-changed',
+        let searchMatchId = Application.searchMatchManager.connect('active-changed',
             Lang.bind(this, this._onActiveMatchChanged));
-        this._collectionId = Application.documentManager.connect('active-collection-changed',
+        let collectionId = Application.documentManager.connect('active-collection-changed',
             Lang.bind(this, this._onActiveCollectionChanged));
 
         this._onActiveSourceChanged();
@@ -174,6 +171,13 @@ const OverviewSearchbar = new Lang.Class({
         this._onActiveMatchChanged();
 
         this._searchEntry.set_text(Application.searchController.getString());
+        this.connect('destroy', Lang.bind(this,
+            function() {
+                Application.sourceManager.disconnect(sourcesId);
+                Application.searchTypeManager.disconnect(searchTypeId);
+                Application.searchMatchManager.disconnect(searchMatchId);
+                Application.documentManager.disconnect(collectionId);
+            }));
     },
 
     createSearchWidgets: function() {
@@ -297,36 +301,12 @@ const OverviewSearchbar = new Lang.Class({
         this._dropdownButton.set_active(true);
     },
 
-    destroy: function() {
-        if (this._sourcesId != 0) {
-            Application.sourceManager.disconnect(this._sourcesId);
-            this._sourcesId = 0;
-        }
-
-        if (this._searchTypeId != 0) {
-            Application.searchTypeManager.disconnect(this._searchTypeId);
-            this._searchTypeId = 0;
-        }
-
-        if (this._searchMatchId != 0) {
-            Application.searchMatchManager.disconnect(this._searchMatchId);
-            this._searchMatchId = 0;
-        }
-
-        if (this._collectionId != 0) {
-            Application.documentManager.disconnect(this._collectionId);
-            this._collectionId = 0;
-        }
-
-        this.parent();
-    },
-
-    show: function() {
+    reveal: function() {
         this._selectAll.enabled = false;
         this.parent();
     },
 
-    hide: function() {
+    conceal: function() {
         this._dropdownButton.set_active(false);
         this._selectAll.enabled = true;
 
